# Release information about zoner

version = "1.4.0"

description = "A DNS zone management web application."
long_description = """\
Zoner is a web application to make management of DNS zone files simple and easy.
The authoritative copy of each domain remains in the original zone file,
which Zoner reads & writes as needed, as opposed to storing domain details
in a database.  This means that zone files can still be edited manually
and Zoner will pick up the changes as necessary.

Zoner features:

* Domain details remain in original zone files, not in a database.
* Zoner reads & writes actual zone files, which can also be safely
modified outside of Zoner.
* Zone serial numbers are incremented automatically when changes are made.
* Zoner can signal bind to reload a zone, via rndc.
* An audit of all zone changes is maintained for each domain.  Any previous
version of a zone file can be inspected and zones can be rolled back to
any previous version.

Requirements:

* Zoner is a Python application built with the TurboGears framework.  Both
Python and TurboGears (version 1.x) are required.
* Zoner requires the easyzone and dnspython Python packages for DNS/zone management.
* Zoner also requires SQLAlchemy, TGBooleanFormWidget and TGExpandingFormWidget Python packages.

(All dependencies should be installed automatically if using setuptools, which will usually be the case for a properly installed TurboGears environment.)


Installation
------------

The easiest way to install Zoner is by using setuptools::

    $ easy_install zoner

Alternatively, install TurboGears then download the Zoner package
and install with::

    $ python setup.py install

Then create a config file.  A template sample-prod.cfg file is included
with the package (or installed alongside the package).  Example::
    
    $ cp /usr/lib/python2.4/site-packages/zoner-1.3.1-py2.4.egg/config/sample-prod.cfg zoner.cfg

Customise the config file, then initialise the database::

    $ tg-admin sql create

Next, create a user to login to the Zoner application with::

    $ zoner_users -c zoner.cfg add

Finally, start the Zoner application::

    $ zoner zoner.cfg

Point your browser at http://localhost:8080/ (or the appropriate host/port
as per your configuration) and you should be able to login.
"""
author = "Chris Miles"
email = "miles.chris@gmail.com"
copyright = "Copyright (c) 2008 Chris Miles"

url = "http://pypi.python.org/pypi/zoner/"
download_url = "http://pypi.python.org/pypi/zoner/"
license = "MIT"



import turbogears
from nose import with_setup
from turbogears import testutil
from zoner.controllers import Root
import cherrypy

def teardown_func():
    """Tests for apps using identity need to stop CP/TG after each test to
    stop the VisitManager thread. See http://trac.turbogears.org/turbogears/ticket/1217
    for details.
    """
    turbogears.startup.stopTurboGears()

cherrypy.root = Root()

def test_method():
    "the index method should return a string called now"
    import types
    result = testutil.call(cherrypy.root.index)
    assert type(result["now"]) == types.StringType
test_method = with_setup(teardown=teardown_func)(test_method)

def test_indextitle():
    "The indexpage should have the right title"
    testutil.createRequest("/")
    response = cherrypy.response.body[0].lower()
    assert "<title>dns zone ui</title>" in response
test_indextitle = with_setup()(test_indextitle)

def test_logintitle():
    "login page should have the right title"
    testutil.createRequest("/login")
    response = cherrypy.response.body[0].lower()
    assert "<title>login</title>" in response
test_logintitle = with_setup()(test_logintitle)



from turbogears import validators
from turbogears.widgets import ListForm, TextField, HiddenField, SubmitButton
from turbogears.widgets import SingleSelectField

from tg_expanding_form_widget.tg_expanding_form_widget import ExpandingForm


class MXNumber(validators.FancyValidator):
    
    messages = {
        'number': 'Please enter a number',
    }
    
    def _to_python(self, value, state):
        
        # This is the hack to support MXPreferenceValidator error
        #   propagation.
        if isinstance(value, list):
            raise validators.Invalid(self.message('number', state),
                          value, state)
        
        try:
            value = float(value)
            if value == int(value):
                return int(value)
            return value
        except ValueError:
            raise validators.Invalid(self.message('number', state),
                          value, state)

class MXPreferenceValidator(validators.FancyValidator):
    def _to_python(self, value, state):
        # print "MXPreferenceValidator value:",value
        rtype = value.get('type', None)
        if rtype and rtype == 'MX':
            try:
                value['preference'] = int(value['preference'])
            except ValueError:
                # This is kind of a hack, but the only way I could
                #   propagate this error back to the field's own validator.
                # Without this the error was being displayed for all
                #   fields in the list...
                value['preference'] = [value['preference']]
                # raise validators.Invalid(
                #     'Please enter a number.',
                #     value, state)
        
        return value
    



class FormSchema(validators.Schema):
    hostname = validators.UnicodeString(not_empty=True, max=256, strip=True)
    type = validators.OneOf(['A', 'CNAME', 'MX', 'NS', 'TXT'])
    preference = MXNumber()
    value = validators.UnicodeString(not_empty=True, max=256, strip=True)


class ExpandingFormSchema(validators.Schema):
    zones = validators.ForEach(
        validators.All(
            FormSchema(),
            MXPreferenceValidator()
        )
    )
    
# class ExpandingFormSchema(validators.Schema):
#     zones = validators.All(validators.ForEach(FormSchema()), MXPreferenceValidator())


class FixedListForm(ListForm):
    template = 'zoner.templates.fixed_list_form'


def createForm():
    hostname = TextField(name='hostname', label=_(u'Hostname'), attrs=dict(size=30))
    rtype = SingleSelectField(
            name='type',
            label=_(u'Type'),
            options=[('A','A'), ('MX','MX'), ('CNAME','CNAME'), ('NS','NS'), ('TXT', 'TXT')],
            attrs=dict(onchange='typeChanged(this);')
            )
    preference = TextField(name='preference', label=_(u'Preference'), attrs=dict(size=4, style='visibility:visible;'))
    value = TextField(name='value', label=_(u'Value'), attrs=dict(size=25))
    afs = ExpandingForm(
            name='zones',
            label=_(u''),
            fields=[hostname, rtype, preference, value],
            # validator=FormSchema()
    )
    zonef = HiddenField(name='zone')
    cancel = SubmitButton(name='cancel', attrs=dict(value=_(u'Cancel'), style="margin-top:1em;"), label=_(u'Cancel all changes'))
    fields = [afs, zonef, cancel]
    
    form = FixedListForm(
        'zoneform',
        fields=fields,
        action='save_data',
        submit_text=_(u'Submit Data'),
        validator=ExpandingFormSchema()
    )
    
    return form


expanding_form = createForm()


# encoding: utf-8

'''breadcrumb_widget
'''

__author__ = 'Chris Miles'
__copyright__ = '(c) Chris Miles 2007'
__id__ = '$Id: breadcrumb_widget.py 297 2007-02-19 16:18:08Z chris $'
__url__ = '$URL: https://hugo.thoh.net/svn/psychofx/Projects/Websites/zoner/frontend/trunk/zoner/zoner/breadcrumb_widget.py $'
__version__ = '1.0'


# ---- Imports ----

# - Python Modules -

# - TurboGears Modules -
from turbogears import widgets


# ---- Classes ----

class BreadcrumbWidget(widgets.Widget):
    template = """
<div class="breadcrumb_trail" xmlns:py="http://purl.org/kid/ns#">
  <span py:for="i,crumb in enumerate(trail)" py:strip=""><span py:if="i > 0" py:content="separator" py:strip="" /><a py:if="crumb['href']" href="${crumb['href']}" py:content="crumb['text']" /><span py:if="not crumb['href']" py:replace="crumb['text']" /></span>
</div>
    """
    params = ["trail", "separator"]
    trail = []      # list of dicts {'text': 'crumb text', 'href': 'crumb link'}
    separator = u' -> '






# -*- coding: utf-8 -*-
"""This module contains functions called from console script entry points."""

import getopt
import os
import sys

from os.path import dirname, exists, join

import pkg_resources

import turbogears
import cherrypy

from zoner.user_manage import user_add, user_delete

cherrypy.lowercase_api = True

class ConfigurationError(Exception):
    pass

def start():
    """Start the CherryPy application server."""
    
    pkg_resources.require("TurboGears >= 1.0.4")
    pkg_resources.require("SQLAlchemy >= 0.3.10")
    pkg_resources.require("easyzone >= 1.2.0")
    pkg_resources.require("TGBooleanFormWidget")
    pkg_resources.require("TGExpandingFormWidget")
    
    setupdir = dirname(dirname(__file__))
    curdir = os.getcwd()
    
    # First look on the command line for a desired config file,
    # if it's not on the command line, then look for 'setup.py'
    # in the current directory. If there, load configuration
    # from a file called 'dev.cfg'. If it's not there, the project
    # is probably installed and we'll look first for a file called
    # 'prod.cfg' in the current directory and then for a default
    # config file called 'default.cfg' packaged in the egg.
    if len(sys.argv) > 1:
        configfile = sys.argv[1]
    elif exists(join(setupdir, "setup.py")):
        configfile = join(setupdir, "dev.cfg")
    elif exists(join(curdir, "prod.cfg")):
        configfile = join(curdir, "prod.cfg")
    else:
        try:
            configfile = pkg_resources.resource_filename(
              pkg_resources.Requirement.parse("zoner"),
                "config/default.cfg")
        except pkg_resources.DistributionNotFound:
            raise ConfigurationError("Could not find default configuration.")

    turbogears.update_config(configfile=configfile,
        modulename="zoner.config")
    
    # Replace tg_flash with my preferred implementation
    from zoner.better_flash import flash, _get_flash
    from turbogears import controllers
    controllers.flash = flash
    controllers._get_flash = _get_flash
    
    from zoner.controllers import Root

    turbogears.start_server(Root())


# ---- zoner_users command handler ----

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg
    


help_message = '''
The help message goes here.
'''


def error(msg=''):
    if msg:
        print >> sys.stderr, "error: %s" % msg
    print >> sys.stderr, sys.argv[0] + " -c file.cfg [options] command"
    print >> sys.stderr, "\t Commands are:"
    print >> sys.stderr, "\t\t add <username> <email> <displayname> <password>"
    print >> sys.stderr, "\t\t delete <username>"
    print >> sys.stderr, "\t for help use --help"
    sys.exit(2)


def user_manage(argv=None):
    config = None
    
    if argv is None:
        argv = sys.argv
    
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "hc:v", ["help", "config="])
        except getopt.error, msg:
            raise Usage(msg)
        
        # option processing
        for option, value in opts:
            if option == "-v":
                verbose = True
            if option in ("-h", "--help"):
                raise Usage(help_message)
            if option in ("-c", "--config"):
                config = value
    
    except Usage, err:
        error(str(err.msg))
    
    if len(args) < 1:
        error()
    
    if config:
        turbogears.update_config(configfile=config, modulename="zoner.config")
    else:
        error('No config file specified (eg: "-c dev.cfg")')
    
    command = args[0]
    
    if command == 'add':
        user_add(*args[1:])
    
    elif command == 'delete':
        user_delete(*args[1:])
    
    else:
        error("Unknown command '%s'" % command)

